# /usr/local/bin/python

"""
Rotate Backups script for DirectAdmin
==============================================

DirectAdmin makes really nice backup tarballs and stores them by default in /home/admin/user_backups. There's just
one little problem. If you have the cron set to make daily backups, each day's backup will overwrite the previous
day's. By running this rotator script one per day shortly *before* your daily backup cron runs, we can save seven
daily backups and an arbitrary number of weekly backups.

Here's what the script will do:

1. Rename DirectAdmin new arrival tarballs to include tarball's mtime date, then move into <username>/daily/ dir.
2. For any daily backups which are more than 7 days old, either move them into weekly, or delete
3. Delete excess backups from weekly dir (in excess of user setting: max_weekly_backups)


This will effectively turn a user_backups dir like this:

user_backups/
   bigco.tar.gz
   mysite.tar.gz


...into this:

user_backups_archive/
   bigco/
      daily/
         bigco.20080101.tar.gz
   mysite/
      daily/
         mysite.20080101.tar.gz



Those daily tarball will continue to pile up for the first week, after which another directory will appear for the 
weekly tarballs as well.



How to install
--------------------------------------------------
1. Place this script somewhere on your server, for example: /usr/local/directadmin/script/rotate_backups.py
2. chmod 700 /usr/local/directadmin/script/rotate_backups.py
3. Add a cron like this -->  0 2 * * * /usr/local/directadmin/scripts/rotate_backups.py > /dev/null

In step three, we added a cronjob for 2am every night. This would be a good setting if for example your backups cron 
runs every night at 3am. It's best to do all your rotating shortly *before* your backups.

Copyright: No copyright, no guarantees -- USE AT YOUR OWN RISK!!!!

Please enjoy :-)


Sean Schertell
sean@datafly.net


"""


#################################################


# User Settings

backups_dir        = '/home/admin/user_backups/'
archives_dir       = '/home/admin/user_backups_archives/'
weekly_backup_day  = 6  # 0-6, Monday-Sunday
max_weekly_backups = 4
exclude_accounts   = [] # list of account names to exclude from backups, ex: ['funkyjoe', 'dirtbag']







#################################################

import os, sys, time, re
from datetime import datetime, timedelta


class Account:
    
    global backups_dir, archives_dir, weekly_backup_day, max_weekly_backups, exclude_accounts
    
    def __init__(self, account_name):
        self.name = account_name
        self.path_to_daily = '%s/%s/daily/' % (archives_dir, self.name)
        self.path_to_weekly = '%s/%s/weekly/' % (archives_dir, self.name)
    
    @classmethod
    def rotate_new_arrivals(self):
        for filename in os.listdir(backups_dir):
            if is_tarball(filename) and filename not in exclude_accounts:
                new_arrival = Backup(os.path.join(backups_dir, filename))
                new_arrival.move_to('daily')
        
    @classmethod
    def collect(object):
        """Return a collection of account objects for all non-excluded accounts in backup dir"""
        accounts = []
        # Append all account names from archives_dir
        for account_name in os.listdir(archives_dir):
            if account_name not in exclude_accounts:
                accounts.append(account_name)
        accounts = sorted(list(set(accounts))) # Uniquify
        return map(Account, accounts)
        
    def rotate_dailies(self):
        seven_days_ago = datetime.today() - timedelta(days = 7)
        for daily in self.get_backups_in('daily'):
            if daily.date < seven_days_ago:
                # This daily is more than seven days old. Move to weekly or delete.
                if daily.date.weekday() == weekly_backup_day:
                    print '%s equals %s' % (daily.date.weekday(), weekly_backup_day)
                    daily.move_to('weekly')
                else:
                    print '%s is not %s' % (daily.date.weekday(), weekly_backup_day)
                    daily.remove()

    def rotate_weeklies(self):
        expiration_date = datetime.today() - timedelta(days = 7 * max_weekly_backups)
        for weekly in self.get_backups_in('weekly'):
            if weekly.date < expiration_date:
                weekly.remove()
        
    def get_backups_in(self, directory):
        backups = []
        path_to_dir = getattr(self, 'path_to_%s' % directory)
        if os.path.isdir(path_to_dir):
            for filename in os.listdir(path_to_dir):
                path_to_file = os.path.join(path_to_dir, filename)
                backups.append(Backup(path_to_file))
        backups.sort()
        return backups
            

class Backup:
    
    global backups_dir, archives_dir, weekly_backup_day, max_weekly_backups, exclude_accounts
        
    def __init__(self, path_to_file):
        """Instantiation also rewrites the filename if not already done (prepends date.)"""
        self.path_to_file = path_to_file
        self.filename = self.format_filename()    
        self.params = self.filename.split('.')
        self.account = self.params[0]        
        self.date = self.get_datetime_obj()
        
    def move_to(self, directory):
        new_filepath = os.path.join(archives_dir, self.account, directory, self.filename)
        try:
            print 'moving to %s' % new_filepath
            os.renames(self.path_to_file, new_filepath)
        except:
            print 'Unable move latest backups into %s/ directory.' % directory
            sys.exit(1)
        
    def remove(self):
        print 'Removing %s' % self.path_to_file
        os.remove(self.path_to_file)
    
        
    def format_filename(self):
        """If this filename hasn't yet been prepended with the date., do that now"""
        # Does the filename start with a date?
        path_parts = self.path_to_file.split('/')
        filename = path_parts[-1]
        parent_dir = '/' + os.path.join(*path_parts[:-1])
        if not re.match('[0-9]{8}', filename.split('.')[1]):
            # No date., rename the file
            self.mtime = time.localtime( os.path.getmtime(self.path_to_file) )
            self.mtime_str = time.strftime('%Y%m%d', self.mtime)
            account = filename.split('.')[0]
            filename = '%s.%s.tar.gz' % (account, self.mtime_str)
            new_filepath = os.path.join(parent_dir, filename)
            print 'Renaming file to %s' % new_filepath
            os.rename(self.path_to_file, new_filepath)
            self.path_to_file = new_filepath
        return filename
            
    def get_datetime_obj(self):
        date_str = self.params[1]
        year = int(date_str[:4])
        month = int(date_str[4:6])
        day = int(date_str[6:8])
        return datetime(year, month, day)
        
    def __cmp__(x, y):
        """For sorting by date"""
        return cmp( x.date, y.date)
        


def is_tarball(filename):
    return filename[-7:] == '.tar.gz'
    
    
        
###################################################



# Make sure backups_dir actually exists
if not os.path.isdir(backups_dir):
    print "Unable to find backups directory: %s" % backups_dir
    sys.exit(1)

# Make sure archives_dir actually exists
if not os.path.isdir(archives_dir):
    try:
        os.mkdir(archives_dir)
    except:
        print "Unable to create archives directory: %s" % archives_dir
        sys.exit(1)
                
# For each account, rotate out new_arrivals, old dailies, old weeklies

Account.rotate_new_arrivals()

for account in Account.collect():
    account.rotate_dailies()
    account.rotate_weeklies()
    
sys.exit(0)




#########################################################
